use DataScience
go


-- can respecify @symbol, @start_date, and @end_date to 
-- modify the data pulled from a database of time series values
declare 
 @symbol varchar(5) = 'GOOGL'
,@start_date date = '2021-01-01'
,@end_date date = '2021-04-30'

-- create #for_all_avg_gain_loss
-- populate row 14 for avg_gain and avg_loss
-- then move avg_gain and avg_loss to row 15

drop table if exists #for_all_avg_gain_loss

-- for avg_gain and avg_loss for row 15
-- and base table for updating
select 
row_number() over (order by date) row_number
,symbol
,[date]
,[close]
,round(isnull([close] 
	- lag([close],1) over (order by date),0),5) change
,round(isnull(gain,0),5) gain
,round(isnull(abs(loss),0),5) loss
,
round(
case
	when row_number <= 13 then null
	when row_number = 14 then 
		avg(round(isnull(gain,0),5)) over
		(order by [date] rows between 13 preceding and current row) 
end 
,5) avg_gain
,
round(
case
	when row_number <= 13 then null
	when row_number = 14 then 
		avg(round(isnull(abs(loss),0),5)) over
		(order by [date] rows between 13 preceding and current row) 
end 
,5) avg_loss
,cast(NULL as real) [relative strength (rs)]
,cast(NULL as real) [rsi]
into #for_all_avg_gain_loss
from
(
-- for gains and losses
select 
symbol
,[date]
,row_number() over (order by [date]) row_number
,[close]
,[close]-(lag([close]) over(order by date)) change
,
case
when ([close]-(lag([close]) over(order by date))) > 0
	then [close]-(lag([close]) over(order by date))
when ([close]-(lag([close]) over(order by date))) = 0
	then 0
end gain
,
case
when ([close]-(lag([close]) over(order by date))) < 0
	then [close]-(lag([close]) over(order by date))
when ([close]-(lag([close]) over(order by date))) = 0
	then 0
end loss
from dbo.stooq_prices
where symbol = @symbol and date >= @start_date and date <= @end_date
) for_14_row_avg_gain_loss

-- move avg_gain and avg_loss for row_number 14
-- to avg_gain and avg_loss for row_number 15
-- and assign null values to avg_gain and avg_loss for row_number 14
update #for_all_avg_gain_loss
set avg_gain = (select avg_gain  from #for_all_avg_gain_loss where row_number = 14),
	avg_loss = (select avg_loss  from #for_all_avg_gain_loss where row_number = 14)
where row_number = 15
update #for_all_avg_gain_loss
set avg_gain = null,
	avg_loss = null
where row_number = 14

-- basic data and computation of first 14 rows
select * from #for_all_avg_gain_loss 
----------------------------------------------------------------------------------------------------

-- declare local variables for Relative Strength
-- and Relative Strength Index (rsi) computation
declare @date_cnt int = (select count(date) from #for_all_avg_gain_loss)
,@row_number int = 15
,@gain_prior money
,@loss_prior money
,@avg_gain_prior money
,@avg_loss_prior money
,@gain_current money
,@loss_current money
,@avg_gain_current money
,@avg_loss_current money
,@rs_prior real
,@rsi_prior real
,@rs_current real
,@rsi_current real

----------------------------------------------------------------------------------------------------

-- start looping through rows after 14th row until last row
-- process rows iteratively two at a time
while @row_number > 14 and @row_number <= 
	(select count(date) from #for_all_avg_gain_loss)  
begin


-- assign values to local variables for prior and current rows
-- initially 15 denotes prior row and 16 denotes current row
-- pairs of rows are successively (15,16), (16,17), (17,18)...
select
 @gain_prior = isnull((select gain from #for_all_avg_gain_loss 
				where row_number = @row_number),0)
,@loss_prior = abs(isnull((select loss from #for_all_avg_gain_loss 
				where row_number = @row_number),0)) 
,@avg_gain_prior = (select avg_gain from #for_all_avg_gain_loss 
				where row_number = @row_number)
,@avg_loss_prior  = abs((select avg_loss from #for_all_avg_gain_loss 
				where row_number = @row_number))
,@gain_current = isnull((select gain from #for_all_avg_gain_loss 
				where row_number = @row_number + 1),0)
,@loss_current = abs(isnull((select loss from #for_all_avg_gain_loss 
				where row_number = @row_number + 1),0))
,@avg_gain_current = (@avg_gain_prior*13 + @gain_current)/14
,@avg_loss_current = (@avg_loss_prior*13 + @loss_current)/14


-- update prior row
update #for_all_avg_gain_loss
set 
	[relative strength (rs)] = @avg_gain_prior/@avg_loss_prior
	,rsi = 
	 case
	 when @avg_loss_prior != 0 then 
		(100 - (100/(1+(@avg_gain_prior/@avg_loss_prior))))
	 else
	 100
	 end
where #for_all_avg_gain_loss.row_number = @row_number

-- update current row
update #for_all_avg_gain_loss
set 
	 avg_gain = ((@avg_gain_prior*13) + @gain_current)/14
	,avg_loss = ((@avg_loss_prior*13) + @loss_current)/14
	,[relative strength (rs)] = 
		(((@avg_gain_prior*13) + @gain_current)/14)
		/(((@avg_loss_prior*13) + @loss_current)/14)
	,rsi = 
	 case
	 when @avg_loss_current != 0 then 
		(100 - (100/(1+(@avg_gain_current/@avg_loss_current))))
	 else
	 100
	 end
where #for_all_avg_gain_loss.row_number = @row_number+1

-- increment @row_number by 1 to get to
-- next to rows to process for rsi
set @row_number = @row_number + 1

end


-- for display of computed RS and RSI values
-- as well as comparison of RSI from T-SQL 
-- code to RSI from Excel
select * 
from #for_all_avg_gain_loss 
where row_number >= 15
order by date

